pub use {
    super::B64Alphabet,
    std::{
        fmt::{self, Write},
        io::{self, Read, ErrorKind},
    },
};

#[derive(Debug)]
pub enum Error {
    Io(io::Error),
    Fmt(fmt::Error),
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Io(e) => write!(f, "{e}"),
            Self::Fmt(e) => write!(f, "{e}"),
        }
    }
}

impl From<io::Error> for Error {
    fn from(value: io::Error) -> Self {
        Self::Io(value)
    }
}

impl From<fmt::Error> for Error {
    fn from(value: fmt::Error) -> Self {
        Self::Fmt(value)
    }
}

impl std::error::Error for Error {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::Fmt(e) => Some(e),
            Self::Io(e) => Some(e),
        }
    }
}

pub struct Encoder<R: Read, W: Write> {
    reader: R,
    writer: W,
    alphabet: B64Alphabet,
}

impl<R: Read, W: Write> Encoder<R, W> {
    pub fn new(reader: R, writer: W, alphabet: B64Alphabet) -> Self {
        Self { reader, writer, alphabet }
    }

    pub fn encode(&mut self) -> Result<(), Error> {
        todo!()
    }
}

impl<R: Read> Encoder<R, String> {
    pub fn output(self) -> String {
        self.writer
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn encode() {
        let mut encoder = Encoder {
            reader: "Hello, World!".as_bytes(),
            writer: String::new(),
            alphabet: B64Alphabet::default(),
        };
        encoder.encode().unwrap();
        assert_eq!(encoder.output(), "SGVsbG8sIFdvcmxkIQ==");
    }
}
